/*============================================================*/
/*============================================================*/
/*==														==*/
/*==					PlayUtils Header					==*/
/*==														==*/
/*============================================================*/
/*============================================================*/


/*========================================================  Prototypes  */

void DoEndGame (void);
void UpdateWorldRecords (void);
void UpdateStats (void);
void InitiateGameOver (void);
void DoSlaveGameOver (void);
void DrawNames (void);
void UpdatePlayerScore (void);
void UpdateOpponentScore (void);
void DoPlayerScores (void);
void DoOpponentScores (void);
void HandlePlayerFoul (void);
void HandleOpponentFoul (void);
Boolean DoNetGameSetUp (void);
void DisplayHoopla (void);
Boolean CantRepeatTournament (void);
Boolean MustProLeagueTournament (void);
Boolean CanWePlay (void);
void PrepareNewGame (void);


/*========================================================  External Variables */

extern playerType		thePlayer;
extern playerType		theOpponent;
extern ballType			theBall;
extern doorType			theDoor;
extern digiDispType		scoreDisplays[2];
extern replayType		*replayData;
extern statType			theStats[10];
extern string31			theNames[10];
extern string31			mostTitlesName, mostPointsName;
extern string31			mostFoulsName, mostCritsName;
extern soundQueue		crowdQueue, incidentalQueue;
extern Str255			thisName, opponentsName;
extern KeyMap			theKeyMap;
extern WindowPtr		mainWndo;
extern RGBColor			RGBBlackColor, RGBWhiteColor;
extern RgnHandle		screenRgn;
extern GrafPtr			offMaskPort;
extern BitMap			offMaskMap;
extern CGrafPort		offCPartsPort, offCWorkPort;
extern CGrafPtr			offCPartsPtr, offCWorkPtr, offCBackPtr;
extern BitMap			offPartsBits, offWorkBits, offBackBits;
extern Rect				offPartsRect, offWorkRect, offMaskRect;
extern Rect				cameraRect, replayRect, colonDest, screenRect;
extern Rect				colonSrc[2];
extern long				timeElapsed, baseTime, wasTime, timePaused, replayFlag;
extern long				mostTitlesDate, mostPointsDate;
extern long				mostFoulsDate, mostCritsDate;
extern short			frameCounter, newFrameCount, lastFrameCount;
extern short			mostTitles, mostPoints, mostFouls, mostCrits;
extern short			primaryMode, displayMode, screenWide, screenHigh, arenaSize;
extern short			displayHCenter, displayVCenter, whichGame, loopsBallHeld;
extern short			playerFouls, opponentFouls, whosGotBall, whichHumanNumber;
extern short			playerTotalFouls, opponentTotalFouls, playerTotalGoals;
extern short			playerTotalCrits, opponentTotalCrits, opponentTotalGoals;
extern short			playerScore, opponentScore, lengthOfApplause;
extern short			leftPlayerNumber, rightPlayerNumber, loopDelay, wasLastSound;
extern short			masterSendSize, slaveSendSize, loopDelay, lengthOfMob;
extern short			droppedPackets, sentPackets, lastSoundPriority;
extern short			spotPoints, spottedPoints, isLeague;
extern short			teaksThreshholds[10];
extern char				playerWonTheGame;
extern Boolean			pausing, quitting, useQD, isColor, leftScoredLast, gameIsOver;
extern Boolean			autoTeamsDialog, leftGoalIsPlayers, madeThePickUp, soundOn;
extern Boolean			netGameInSession, imTheMaster, playerJustScored, enableAnnouncer;
extern Boolean			canReplay, replayGoals, replayFouls, replayOnR, replaySomething;
extern Boolean			disableBoardCursor, smsActivated, newGame, theyAreMajor;
extern Boolean			wasBrakeOn, drawThisFrame, splashIsUp;
extern Boolean			soundLoaded[kMaxNumberOfSounds];

